// Beispiel 4: Ansteuerung einer gemultiplexten
// 7-Segment-Anzeige, mittels Inkrement und Dekrement Tastern
// von den 8 Stellen werden 2 als Hex-Ziffern genutzt um
// den Wertebereich eines Bytes darzustellen

`include "utils/debouncer.v"  // der Debouncer aus Bsp. 3, ausgelagert


module main (
 input  wire CLK_in,
 input  wire KeyA,
 input  wire KeyB,
 output reg  [7:0] seg,	   // Ziffern-Segmente
 output reg  [7:0] digit	// Ziffern-Position im 8-stelligen Display
);
 wire Key_inc;							// entprelltes Signal fuer INC
 wire Key_dec;	   					// entprelltes Signal fuer DEC
 reg  [7:0] myByte;
 reg  [15:0] countdown;				// Zaehler für Display-Multiplexerei
 reg  [2:0]  Pos;						// Ziffernposition, Wertebereich {0 bis 7}

 // je eine Instanz der Enprellung pro Taster:
 PushButton_Debouncer DebounceDec (.clk(CLK_in), .PB(KeyB), .PB_down(Key_dec));
 PushButton_Debouncer DebounceInc (.clk(CLK_in), .PB(KeyA), .PB_down(Key_inc));
 
 always @ (posedge CLK_in) begin
 if (Key_dec) myByte <= myByte - 1;
 if (Key_inc) myByte <= myByte + 1;
 countdown = countdown - 1;
 if (countdown == 0) begin
   digit <= ~(1 << Pos);		// activate next digit
	
	case(Pos)
	 0: seg <= 8'hFF;	// Ziffern 0 bis 5 hier ungenutzt
	 1: seg <= 8'hFF;
	 2: seg <= 8'hFF;
	 3: seg <= 8'hFF;
	 4: seg <= 8'hFF;
	 5: seg <= 8'hFF;
	 6: seg <= bin2hex(myByte[7:4]);  // hoeherwertiges Nibble
	 7: seg <= bin2hex(myByte[3:0]);  // niederwertiges Nibble
	endcase 
	
	Pos <= Pos + 1;
 end 
end 
 
// ----------
// Umwandlung binäre 4 Bit Zahl in Hex-Ziffer 
// ausgelagert in eine Funktion
function [7:0] bin2hex (input [3:0] x);
begin 
  case (x)
   4'h0: bin2hex =8'd192;
	4'h1: bin2hex =8'd249;
	4'h2: bin2hex =8'd164;
	4'h3: bin2hex =8'd176;
	4'h4: bin2hex =8'd153;
	4'h5: bin2hex =8'd146;
	4'h6: bin2hex =8'd130;
	4'h7: bin2hex =8'd248;
	4'h8: bin2hex =8'd128;
	4'h9: bin2hex =8'd144;
	4'hA: bin2hex =8'd136;
	4'hB: bin2hex =8'd131;
	4'hC: bin2hex =8'd198;
	4'hD: bin2hex =8'd161;
	4'hE: bin2hex =8'd134;
	4'hF: bin2hex =8'd142;
  endcase
end  
endfunction 
 
endmodule // Ende Hauptmodul
